#!/bin/sh
######################################################################
#
#   Module:  rv.py
#   Version: $Id: rv.py,v 1.5 2003/11/14 05:54:10 jsc Exp $
#   Author:  Jeffrey Clement <jclement@bluesine.com>
#   Targets: Win32, Unix
#   Web:     http://jclement.ca/
#
# The recursive image viewer.  Basically just a quick way to do slide
# shows when your images exist in many directories.  The slide show is 
# always fullscreen and lets you tweak things like timing, looping, 
# randomness and titles.  Very simple.
#
# -------------------------------------------------------------------
#
# $Log: rv.py,v $
# Revision 1.5  2003/11/14 05:54:10  jsc
# *** empty log message ***
#
# Revision 1.4  2003/11/01 15:39:14  jsc
# *** empty log message ***
#
# Revision 1.3  2003/11/01 15:03:47  jsc
# *** empty log message ***
#
# Revision 1.2  2003/10/31 22:14:59  jsc
# *** empty log message ***
#
# Revision 1.1.1.1  2003/10/31 21:35:05  jsc
# init
#
# -------------------------------------------------------------------
#
# Copyright (c) 2003, Jeffrey Clement All rights reserved. 
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions 
# are met: 
#
# * Redistributions of source code must retain the above copyright notice, 
#   this list of conditions and the following disclaimer. 
# * Redistributions in binary form must reproduce the above copyright 
#   notice, this list of conditions and the following disclaimer in the 
#   documentation and/or other materials provided with the distribution. 
# * Neither the name of the Bluesine nor the names of its contributors 
#   may be used to endorse or promote products derived from this software 
#   without specific prior written permission. 
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED 
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##########################################################################

DISPLAY_RESOLUTION = (1024,768)

##########################################################################

import os
import re
import sys
import glob
import getopt
import random
from random import shuffle

try: 
    import pygame
    from pygame.locals import *
except:
    print "requires pygame: http://pygame.seul.org"
    sys.exit(0)

def recursive_list(dir):
    out = []
    dc = glob.glob(os.path.join(dir,'*'))
    dc.sort()
    for f in dc:
        if os.path.isdir(f):
            out += recursive_list(f)
        else:
            out.append(f)
    return out

def isImage(file):
    if os.path.splitext(file)[-1].lower() in ['.jpg','.bmp','.gif','.png','.jpeg']:
        return 1
    return 0

def usage():
    print "*"*78
    print "rv.py -- The recursive image viewer"
    print ""
    print "Copyright (C) 2003 Jeffrey Clement"
    print "This is free software; see the source for copying conditions.  There is NO"
    print "warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE."
    print ""
    print "usage: rv.py [OPTION]... [DIR]"
    print " Options:"
    print "  -h             Show this help screen"
    print "  -r             Randomly pick images"
    print "  -l             Loop when run out of images"
    print "  -t             Show titles"
    print "  -d <seconds>   Automatic transition every <seconds> seconds"
    print ""
    print "*"*78
    sys.exit(1)
                                            

def slideshow(dir, delay, random, loop, titles):
    files = recursive_list(dir)
    files = filter(isImage, files) 
    file_pos = 0
    if random:
        shuffle(files)
    # make sure we have at least some files!
    if len(files) == 0: 
        print "No files!"
        sys.exit(0)
    # initialize video mode
    scr = pygame.display.set_mode(DISPLAY_RESOLUTION, FULLSCREEN)
    pygame.font.init()
    font = pygame.font.Font(None, 20)
    while 1:
        if file_pos >= len(files):
            if loop: file_pos=0
            else:    return
        elif file_pos < 0: file_pos=0
        # move image to list of viewed images
        img_fn = files[file_pos]
        # load image
        try:
            img = pygame.image.load(img_fn).convert()
        except Exception, e:
            print e
        # if image larger than display, scale it down
        factor = max(
                1.0 * img.get_size()[0] / scr.get_size()[0],
                1.0 * img.get_size()[1] / scr.get_size()[1] 
                )
        if factor > 1:
            ns = map(int,[img.get_size()[0]/factor,img.get_size()[1]/factor])
            img = pygame.transform.scale(img, ns) # new SIZe
        # display centered on screen
        pos = [
            (scr.get_size()[0] - img.get_size()[0])/2,
            (scr.get_size()[1] - img.get_size()[1])/2
            ]
        scr.fill((0,0,0))
        scr.blit(img, pos)
        # draw titles if necessary
        if titles:
            t = font.render(img_fn, 1, (0,0,0))
            if t.get_size()[0] > scr.get_size()[0]: break
            pos = [
                scr.get_size()[0] - t.get_size()[0] - 5 + 1,
                scr.get_size()[1] - t.get_size()[1] - 5 + 1
                ]
            scr.blit(t, pos) 
            t = font.render(img_fn, 1, (255,255,0))
            if t.get_size()[0] > scr.get_size()[0]: break
            pos = [
                scr.get_size()[0] - t.get_size()[0] - 5,
                scr.get_size()[1] - t.get_size()[1] - 5
                ]
            scr.blit(t, pos) 
        pygame.display.update()
        # wait for input / timeout
        timetochange = 0
        if delay > 0:
            timeleft = 1000 * delay
        while timetochange==0:
            pygame.time.delay(10)
            if delay > 0:
                timeleft -= 10
                if timeleft < 0: 
                    timetochange=1        
                    file_pos += 1
            for event in pygame.event.get():
                if event.type is QUIT: return
                if event.type is KEYDOWN:
                    if event.key == K_q: return
                    if event.key == K_ESCAPE: return
                    if event.key == K_SPACE: 
                        timetochange=1
                        file_pos += 1

if __name__=='__main__':
    try: opts, args = getopt.getopt(sys.argv[1:], 'hlrtd:')
    except getopt.GetoptError, e: usage()

    if len(args) <> 1:
        usage()

    delay = 0
    random = 0
    loop = 0
    titles = 0
    dir = args[0]

    for k,v in opts:
        if k == '-d': delay = int(v)
        elif k == '-r': random = 1
        elif k == '-l': loop = 1
        elif k == '-t': titles = 1
        else: usage()

    slideshow(dir, delay, random, loop, titles)
    
    
